@@ -65,6 +65,9 @@ pub use store::OpendalStore;
 
 mod utils;
 
+#[cfg(feature = "services-s3")]
+mod amazon_s3;
+
 // Make sure `send_wrapper` works as expected
 #[cfg(all(feature = "send_wrapper", test))]
 mod assert_send {
